@@ -19,7 +19,6 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "greeterapp.h"
-#include "authenticator.h"
 #include "kscreensaversettingsbase.h"
 #include "lnf_integration.h"
 #include "noaccessnetworkaccessmanagerfactory.h"
@@ -33,10 +32,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // KDE
 #include <KAuthorized>
+#include <KConfigPropertyMap>
 #include <KCrash>
-#include <KDeclarative/ConfigPropertyMap>
 #include <KDeclarative/KQuickAddons/QuickViewSharedEngine>
 #include <KDeclarative/QmlObjectSharedEngine>
+#include <KLocalizedContext>
 #include <KWindowSystem>
 #include <kdeclarative/kdeclarative.h>
 
@@ -65,7 +65,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QQuickItem>
 #include <QQuickView>
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
+#endif
 // Wayland
 #include <wayland-client.h>
 #include <wayland-ksld-client-protocol.h>
@@ -76,6 +80,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 //
 #include <xcb/xcb.h>
 
+#include "pamauthenticator.h"
+
 // this is usable to fake a "screensaver" installation for testing
 // *must* be "0" for every public commit!
 #define TEST_SCREENSAVER 0
@@ -95,10 +101,33 @@ void disableDrKonqi()
 // that would enable drkonqi
 Q_CONSTRUCTOR_FUNCTION(disableDrKonqi)
 
+// Verify that a package or its fallback is using the right API
+bool verifyPackageApi(const KPackage::Package &package)
+{
+    if (package.metadata().value("X-Plasma-APIVersion", QStringLiteral("1")).toInt() >= 2) {
+        return true;
+    }
+
+    if (!package.filePath("lockscreenmainscript").contains(package.path())) {
+        // The current package does not contain the lock screen and we are
+        // using the fallback package. So check to see if that package has
+        // the right version instead.
+        if (package.fallbackPackage().metadata().value("X-Plasma-APIVersion", QStringLiteral("1")).toInt() >= 2) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 class FocusOutEventFilter : public QAbstractNativeEventFilter
 {
 public:
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     bool nativeEventFilter(const QByteArray &eventType, void *message, long int *result) override
+#else
+    bool nativeEventFilter(const QByteArray &eventType, void *message, qintptr *result) override
+#endif
     {
         Q_UNUSED(result)
         if (qstrcmp(eventType, "xcb_generic_event_t") != 0) {
@@ -120,15 +149,13 @@ UnlockApp::UnlockApp(int &argc, char **argv)
     , m_testing(false)
     , m_ignoreRequests(false)
     , m_immediateLock(false)
+    , m_authenticator(new PamAuthenticator("kde", KUser().loginName(), this))
     , m_graceTime(0)
     , m_noLock(false)
     , m_defaultToSwitchUser(false)
     , m_wallpaperIntegration(new WallpaperIntegration(this))
     , m_lnfIntegration(new LnFIntegration(this))
 {
-    m_authenticator = createAuthenticator();
-    // It's a queued connection to give the QML part time to eventually execute code connected to Authenticator::succeeded if any
-    connect(m_authenticator, &Authenticator::succeeded, this, &QCoreApplication::quit, Qt::QueuedConnection);
     initialize();
 
     if (QX11Info::isPlatformX11()) {
@@ -160,11 +187,6 @@ UnlockApp::~UnlockApp()
     }
 }
 
-Authenticator *UnlockApp::createAuthenticator()
-{
-    return new Authenticator(AuthenticationMode::Direct, this);
-}
-
 void UnlockApp::initialize()
 {
     // set up the request ignore timeout, so that multiple requests to sleep/suspend/shutdown
@@ -183,6 +205,10 @@ void UnlockApp::initialize()
     if (!KScreenSaverSettingsBase::theme().isEmpty()) {
         package.setPath(KScreenSaverSettingsBase::theme());
     }
+    if (!verifyPackageApi(package)) {
+        qCWarning(KSCREENLOCKER_GREET) << "Lockscreen QML outdated, falling back to default";
+        package.setPath(QStringLiteral("org.kde.breeze.desktop"));
+    }
 
     m_mainQmlPath = package.fileUrl("lockscreenmainscript");
 
@@ -301,10 +327,11 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
         view->setGeometry(geo);
     });
 
-    // first create KDeclarative, to be sure that it created a KIO Network Factory
-    KDeclarative::KDeclarative declarative;
-    declarative.setDeclarativeEngine(view->engine());
-    declarative.setupBindings();
+    view->engine()->rootContext()->setContextObject(new KLocalizedContext(view->engine()));
+    auto oldFactory = view->engine()->networkAccessManagerFactory();
+    view->engine()->setNetworkAccessManagerFactory(nullptr);
+    delete oldFactory;
+    view->engine()->setNetworkAccessManagerFactory(new NoAccessNetworkAccessManagerFactory);
 
     if (!m_testing) {
         if (QX11Info::isPlatformX11()) {
@@ -322,6 +349,13 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
 
     // engine stuff
     QQmlContext *context = view->engine()->rootContext();
+    connect(view->engine(), &QQmlEngine::quit, this, [this]() {
+        if (m_authenticator->isUnlocked()) {
+            QCoreApplication::quit();
+        } else {
+            qCWarning(KSCREENLOCKER_GREET) << "Greeter tried to quit without being unlocked";
+        }
+    });
 
     context->setContextProperty(QStringLiteral("kscreenlocker_userName"), m_userName);
     context->setContextProperty(QStringLiteral("kscreenlocker_userImage"), m_userImage);
@@ -331,6 +365,15 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
     context->setContextProperty(QStringLiteral("defaultToSwitchUser"), m_defaultToSwitchUser);
     context->setContextProperty(QStringLiteral("config"), m_lnfIntegration->configuration());
 
+    loadWallpaperPlugin(view);
+    if (auto object = view->property("wallpaperGraphicsObject").value<KDeclarative::QmlObjectSharedEngine *>()) {
+        // initialize with our size to avoid as much resize events as possible
+        object->completeInitialization({
+            {QStringLiteral("width"), view->width()},
+            {QStringLiteral("height"), view->height()},
+        });
+    }
+
     view->setSource(m_mainQmlPath);
     // on error, load the fallback lockscreen to not lock the user out of the system
     if (view->status() != QQmlComponent::Ready) {
@@ -343,12 +386,6 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
     }
     view->setResizeMode(KQuickAddons::QuickViewSharedEngine::SizeRootObjectToView);
 
-    // overwrite the factory set by kdeclarative
-    auto oldFactory = view->engine()->networkAccessManagerFactory();
-    view->engine()->setNetworkAccessManagerFactory(nullptr);
-    delete oldFactory;
-    view->engine()->setNetworkAccessManagerFactory(new NoAccessNetworkAccessManagerFactory);
-
     QQmlProperty lockProperty(view->rootObject(), QStringLiteral("locked"));
     lockProperty.write(m_immediateLock || (!m_noLock && !m_delayedLockTimer));
 
@@ -367,19 +404,11 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
     // verify that the engine's controller didn't change
     Q_ASSERT(dynamic_cast<NoAccessNetworkAccessManagerFactory *>(view->engine()->networkAccessManagerFactory()));
 
-    loadWallpaperPlugin(view);
-    if (auto object = view->property("wallpaperGraphicsObject").value<KDeclarative::QmlObjectSharedEngine *>()) {
-        // initialize with our size to avoid as much resize events as possible
-        object->completeInitialization({
-            {QStringLiteral("width"), view->width()},
-            {QStringLiteral("height"), view->height()},
-        });
-    }
-
     if (KWindowSystem::isPlatformWayland()) {
         if (auto layerShellWindow = LayerShellQt::Window::get(view)) {
             layerShellWindow->setExclusiveZone(-1);
             layerShellWindow->setLayer(LayerShellQt::Window::LayerOverlay);
+            layerShellWindow->setDesiredOutput(screen);
         }
     }
 
